/****************************************************************
 * Firmware of the TI TRF7970A Booster Pack CE TYPE A Rev. 1.0
 *
 * Copyright (C) 2012 Texas Instruments, Inc.
 *
 * DESCRIPTION: The TRF7970A is an integrated analog front end and
 * data framing system for a 13.56 MHz NFC/RFID reader system.
 * Built-in programming options make it suitable for a wide range
 * of applications both in proximity and vicinity RFID systems.
 * The reader is configured by selecting the desired protocol in
 * the control registers. Direct access to all control registers
 * allows fine tuning of various reader parameters as needed.
 *
 * The TRF7970A is interfaced to a MSP430G2553 microcontroller
 * through the SPI (serial) interface using a hardware USART.
 * The MCU is the master device and initiates all communication
 * with the reader.
 *
 *
 *
 * AUTHOR:    Embedded RF    DATE: June 13, 2012
 *
 * CHANGES:
 * REV.    DATE         WHO    DETAIL
 * 1.0   June 13, 2012  ERF    Orginal Version
 * 2.0   Aug 29, 2012   JH     Modified to take in I2C data to fill in pairing information.
 *
 *
 * BUILT WITH:
 * IAR for MSP430, v5.40.1 (5.40.1.50328)
 * (C) Copyright Texas Instruments, 2012. All rights reserved.
 *****************************************************************
 *****************************************************************
 * All software and related documentation is provided "AS IS" and
 * without warranty or support of any kind and Texas Instruments
 * expressly disclaims all other warranties, express or implied,
 * including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose. Under no
 * circumstances shall Texas Instruments be liable for any
 * incidental, special or consequential damages that result from
 * the use or inability to use the software or related
 * documentation, even if Texas Instruments has been advised of
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by
 * Texas Instruments is distributed as "freeware". You may use
 * and modify this software without any charge or restriction.
 * You may distribute to others, as long as the original author
 * is acknowledged.
 ****************************************************************/
//
//===============================================================
#include <msp430.h>                         // processor specific header
#include <stdlib.h>                         // general purpose standard library
#include <stdio.h>                          // standard input/output header
#include "mcu.h"
#include "trf796x.h"
#include "types.h"
#include "spi.h"
#include "trf796xBoosterPack.h"
#include "parallel.h"

void I2c_setup(void);
//===============================================================

typedef enum {
	FALSE_s = 0, TRUE_s
} boolean_state;

u08_t buf[130]; // Allocate 130 bytes of RAM
u08_t i_reg = 0x01; // interrupt register
u08_t irq_flag = 0x00;
u08_t rx_error_flag = 0x00;
s08_t rxtx_state = 1; // used for transmit receive byte count
volatile unsigned char g_i2c_busy_flag = FALSE_s;
unsigned char *PRxData; // Pointer to RX data
unsigned char *PTxData;
unsigned char RXByteCtr = 0;
unsigned char TXByteCtr = 0;

unsigned char RXBuffer[55];

typedef enum {
	HOST_READING = 0, HOST_WRITING, HOST_IDLE, HOST_UPDATE
} g_message_direction_t;

volatile g_message_direction_t g_message_direction = HOST_IDLE;
volatile unsigned char g_i2c_packet_length = 0;
volatile unsigned char g_i2c_register = 0;
volatile unsigned char g_i2c_packet_decode_flag = FALSE_s;
unsigned char g_i2c_checksum = 0;

//
// MSP430G2553 Register LSB R/nW Flag
//
#define		READ_FLAG_MASK			0x01

//
// MSP430G2553 Register Map
//
#define		FIRMWARE_VER_REG		0x00
#define		FIRMARE_STATUS_REG		0x01
#define 	FIRMARE_UPDATE_REG		0x02
//
// Registers 03h-0fh reserved for future use
//
#define 	BT_ADDRESS_REG			0x10
#define 	DEVICE_NAME_REG			0x11
#define 	DEVICE_NAME_LENGTH_REG	0x12
#define		ADD_NDEF_REG			0x13
#define 	NDEF_LENGTH				0x14
//
// Registers 15h-7fh reserved for future use
//

#define		BT_ADDR_BUF_LENGTH		06
#define		DEVICE_NAME_BUF_LENGTH	30
#define 	NDEF_BUF_LENGTH			50

unsigned char g_firmware_version_reg;
unsigned char g_working_mode;
unsigned char g_application_validate_mark;
unsigned char g_bt_address_reg[BT_ADDR_BUF_LENGTH] = { 0x11, 0x22, 0x33, 0x44,0x55, 0x66 };
unsigned char g_device_name_reg[DEVICE_NAME_BUF_LENGTH]= {'D','e','v','i','c','e','N','a','m','e','\n'};
unsigned char g_device_name_length_reg = 0x0B;

//*****************************************************************************
//
// Enable the BT pairing response that will be sent as the NDEF Message.
// Enabled = 1, Disabled = 0
//
#define TRANSMIT_SIMPLE_RESPONSE				1
#define TRANSMIT_STATIC_HANDOVER_RESPONSE		0
//*****************************************************************************

//*****************************************************************************
//
// Check for correct definition of RF_DAUGTHER_TRF79X0
//
#if (TRANSMIT_SIMPLE_RESPONSE && TRANSMIT_STATIC_HANDOVER_RESPONSE)
#error "Only one BT pairing response can be defined at the same time."
#elif (!(TRANSMIT_SIMPLE_RESPONSE || TRANSMIT_STATIC_HANDOVER_RESPONSE))
#error "Define the BT Pairing response to send as the NDEF message."
#endif

//
// BSL Working Modes
//
#define 	APP_MODE		1
#define		BOOT_MODE		0

//===============================================================
//
//===============================================================
//
// Macro definitions
//

/*! Module is busy waiting for IRQ */
#define INT_BUSY            BIT0
/*! RX detected and in buffer */
#define INT_RX_COMPLETE     BIT1
/*! Critical error during RFID processing */
#define INT_CRITICAL_ERROR  BIT2
/*! Communication error (protocol error) */
#define INT_COMM_ERROR      BIT7
/*! TX Completed */
#define INT_TX_COMPLETE     BIT3

/*! Interrupt, error, flow flags */
volatile u08_t u8InterruptFlags;

#define IRQ_IS_SET()            (P2IN & IRQ_PIN)
#define SLAVESELECT_HIGH()      {SPI_SS_OUT |= SPI_SS_PIN;}
#define SLAVESELECT_LOW()       {SPI_SS_OUT &= ~SPI_SS_PIN;}

typedef enum {
	POWER_OFF_STATE = 0, /* Default state after reset */
	SENSE_STATE, /* Target is waiting for 14443B transponder emulation command from reader */
	EMULATION_STATE, /* Target is in card emulation mode */
	NDEF_MESSAGE_STATE, /* Target has sent the NDEF Message */
	DESELECTED_STATE
/* Target has been deselected from the reader */
} NFC_state_t;
NFC_state_t NFC_State;

#define FIFO_LEN	127

u08_t state_250ms_counter = 0;

/*! Buffer pointer */
static u08_t * buf_ptr;

/******************************************************************************
 * @brief  Protocol Error handler
 *  Called when the TRF7970 detects a protocol error
 *
 * @return    None
 *****************************************************************************/
void NFC_IRQ_PROTOCOL_ERR(void) {
	u8InterruptFlags = INT_COMM_ERROR;
	Trf797xInitialSettings();
}

/******************************************************************************
 * @brief  TX handler
 *  Called when the TRF7970 detects a Transmission complete
 *
 * @return    None
 *****************************************************************************/
void NFC_IRQ_TX(void) {
	// Reset FIFO with an extra dummy clock after last byte has been read out
	buf[0] = 0x00;
	RADIO_WRITERAW(RESET|BIT7, buf, 1);
	u8InterruptFlags = INT_TX_COMPLETE;
}

/******************************************************************************
 * @brief  TX handler
 *  Called when the TRF7970 detects a Transmission complete
 *
 * @return    None
 *****************************************************************************/
void NFC_IRQ_SDD(void) {

	// Reset FIFO with an extra dummy clock after last byte has been read out
	buf[0] = 0x00;
	RADIO_WRITERAW(RESET|BIT7, buf, 1);

}


/******************************************************************************
 * @brief  RX handler
 *  Called when the TRF7970 detects a data Reception
 *  It puts the data in buffers which will be passed to upper layers
 *
 * @param Fifo_Int 1: Interrupted by FIFO, 0: RX complete
 *
 * @return    None
 *****************************************************************************/
void NFC_IRQ_RX(u08_t Fifo_Int) {
	u08_t Reg;

	buf_ptr = &buf[0];
	RADIO_READSINGLE(FIFO_STATUS, &Reg);
	//determine the number of bytes left in FIFO
	Reg = 0x7F & Reg;

	// Get all bytes from FIFO and put them in the buffer
	RADIO_READCONT(FIFO, buf_ptr, (Reg));
	u8InterruptFlags = INT_RX_COMPLETE;
}

/******************************************************************************
 * @brief Process a TRF797x IRQ
 * Called when an IRQ is detected, calls the appropiate function from NFC layer
 *
 * @param IRQ_Stat value read from IRQ register
 *
 * @return  none
 *****************************************************************************/
void Radio_Process_IRQ(u08_t IRQ_Stat) {
	if (IRQ_Stat & BIT5) {
		//
		// FIFO is high/low
		//
		__no_operation();
			//NFC_IRQ_FIFO();
//	} else if (IRQ_Stat & BIT0) {
//        NFC_IRQ_CRITICAL_ERR();
	} else if (IRQ_Stat & BIT4) {
		NFC_IRQ_PROTOCOL_ERR();
//	} else if (IRQ_Stat & BIT2) {
//        NFC_IRQ_RF();
	} else if (IRQ_Stat & BIT3) {
		//IRQ SDD Complete interrupt
        NFC_IRQ_SDD();
	} else if (IRQ_Stat & BIT6) {
		// Reception detected, pass if FIFO RX or end of RX
		NFC_IRQ_RX(IRQ_Stat & BIT5);
	} else if (IRQ_Stat & BIT7) {
		NFC_IRQ_TX();
	}

}

/******************************************************************************
 * @brief Waits for a radio IRQ
 * This function will lock the mcu while waiting for IRQ. Note that the
 * microcontroller is not expected to do anything else in BSL
 *
 * @return  none
 *****************************************************************************/
void Radio_Wait_IRQ(void) {
	u08_t Register[4];
	static unsigned char retry_counter = 0;

	// Wait for an IRQ forever
	while (!IRQ_IS_SET())
	{
//		TRF_DISABLE;
		__bis_SR_register(LPM3_bits + GIE);
	}

	if (g_i2c_busy_flag  == TRUE_s) {
		NFC_State = POWER_OFF_STATE;
		return;
	}

	do {
		// Clear Buffer
		Register[0] = IRQ_STATUS;
		Trf797xReadCont(Register, 2); /* IRQ status register address */
//		Trf796xReadSingle(&Register[0], 1);

		Register[2] = NFC_TARGET_PROTOCOL;
		Trf797xReadSingle(&Register[2], 1);

#if CARD_EMULATION_TYPE_A
		if (Register[2] == 0xC9) {
#endif
#if CARD_EMULATION_TYPE_B
			if (Register[2] == 0xC5) {
#endif
			retry_counter = 0;
			Radio_Process_IRQ(Register[0]);
			 break;
		}
		//
		// Reset the TRF7970 when :
		// 1. Target Protocol Register has the RF_h and RF_l enabled but incorrect bit rate ( not 106 kbps)
		// 2. Target Protocol Register
		//
		else if (((Register[2]) & 0x0F) != 0x00 || Register[2] == 0xC0 || Register[2] == 0x40 || Register[1] != 0x3E) {
			Trf797xInitialSettings();
			NFC_State = SENSE_STATE;
		}
		else if( Register[2] == 0x80)
		{
			Trf797xInitialSettings();
		}
		else
		{
			//
			// Increment retry counter, once it reaches 10 then reset
			//
			retry_counter++;
			if(retry_counter == 10)
			{
				retry_counter = 0;
				Trf797xInitialSettings();
			}
			NFC_State = SENSE_STATE;
		}

	} while (IRQ_IS_SET() && g_i2c_busy_flag == FALSE_s);

	return;
}


void NFCSendCRC(u08_t *pbuf, u08_t crc_enable) {
	u08_t total_length, length, i;
	u08_t tx_buf[FIFO_LEN + 3];

	u8InterruptFlags = INT_BUSY;

	total_length = *pbuf;
	length = total_length;
	if (length > FIFO_LEN)
		length = FIFO_LEN;

	tx_buf[0] = RESET | BIT7;
	if(crc_enable == TRUE_s)
	{
		// First, clear FIFO with a reset
		tx_buf[1] = TRANSMIT_CRC | BIT7; // TX command
	}
	else
	{
		// First, clear FIFO with a reset
		tx_buf[1] = TRANSMIT_NO_CRC | BIT7; // TX command
	}

	tx_buf[2] = TX_LENGTH_BYTE_1 | BIT5; // Write cont TXLength
	tx_buf[3] = (u08_t) (total_length >> 4);
	tx_buf[4] = (u08_t) (total_length << 4);

	for (i = 1; i <= length; i++)
		tx_buf[i + 4] = pbuf[i];

//	RADIO_WRITERAW(RESET | BIT7,(u08_t *) tx_buf, length+4);
	ParallelRawWrite(tx_buf,length + 5);

	do {
		// Wait for TX Complete interrupt
		Radio_Wait_IRQ();
	} while (u8InterruptFlags != INT_TX_COMPLETE && NFC_State == EMULATION_STATE);
}

void NFCSend(u08_t *pbuf)
{
	NFCSendCRC(pbuf,TRUE_s);
}

u08_t NFC_ProcessReceivedData(u08_t * buffer) {
	u16_t command;
	u08_t ret = 0x00; //RX_PACKET_ONGOING
	volatile u08_t index_temp;
	static u08_t card_identifier;
	static u16_t selected_file;
	u08_t card_block_number;

	command = buffer[0];

	//
	// Mask off BIT7:5 + BIT1 == 0xA2 => I-Block PCB Command
	// 						OR
	// Mask off BIT7:5 + BIT1 == 0x02 => R-Block PCB Command
	//
	if( (command & 0xE2) == 0xA2 || (command & 0xE2) == 0x02)
	{
		card_block_number = command & 0x01;
		//
		// Remove the Block Number from the command
		//
		command &= 0xFE;
	}

	// Check if the command is a PPS command
	if( (command & 0xF0) == 0xD0)
	{
		// Store the Card Identifier(CID) and make the command a PPS command (0xD0)
		card_identifier = command&0x0F;
		command = 0xD0;
	}

	switch (command)
	{
	//
	// Type A Commands
	//

	//
	// Received a REQA / SENS_REQ
	//
	case 0x26:
	    // Register 01h. ISO Control Register
	    buf[0] = 0xA4; // Type A
	    //
	    // 106 kbps Tag Emulator 14443A mode
	    //
	    RADIO_WRITESINGLE(ISO_CONTROL, buf);

		buffer[0] = 0x02;	// Length of packet
		// BIT15:12 RFU
		// BIT11:8 Proprietary - 0x03
		// BIT7:6  NFCID Length - 00 - Single, 01 - Double, 11 - Triple
		// BIT5 RFU
		// BIT4:0 Bit frame anti-collision - Bit 1
		buffer[1] = 0x04;
		buffer[2] = 0x00;
		NFCSendCRC(&buffer[0],FALSE_s); //Send complete UID (93 20) / SAK ( 93 70 + UID)

		break;

	//
	// Received a WuPA / ALL_REQ
	//
	case 0x52:
		// Reset FIFO with an extra dummy clock
		buffer[0] = 0x00;
		RADIO_WRITERAW(RESET|BIT7, buffer, 1);

		buffer[0] = 0x02;	// Length of packet
		// BIT15:12 RFU
		// BIT11:8 Proprietary - 0x03
		// BIT7:6  NFCID Length - 00 - Single, 01 - Double, 11 - Triple
		// BIT5 RFU
		// BIT4:0 Bit frame anti-collision - Bit 1
		buffer[1] = 0x04;
		buffer[2] = 0x00;
		NFCSendCRC(&buffer[0],FALSE_s); //Send complete UID (93 20) / SAK ( 93 70 + UID)
		break;

	//
	// Received a anticollision command # 1
	//
	case 0x93:
		//
		// Cascade level 1
		//
		if(buffer[1] == 0x20)
		{
			buffer[0] = 0x05;	//Length of packet
			//
			// Clear Buffer[5]
			//
			buffer[5] = 0x00;
		    for ( index_temp=0; index_temp < 4; index_temp++)
		    {
		    	buffer[index_temp+1] = TargetCID[index_temp];
		    	buffer[5] ^= TargetCID[index_temp];
		    }
		    NFCSendCRC(&buffer[0],FALSE_s);
		}
		//
		// PCD Selected the TRF7970 based on the UID
		//
		else if( buffer[1] == 0x70)
		{
		    // Register 01h. ISO Control Register
		    buf[0] = 0x24; // Type A
		    //
		    // 106 kbps Tag Emulator 14443A mode
		    //
		    RADIO_WRITESINGLE(ISO_CONTROL, buf);
			buffer[0] = 0x01;	//Length of packet
			// BIT5 - UID Complete, PICC Compliant with ISO / IEC 14443-4
			buffer[1] = 0x20;	// SAK
			NFCSend(&buffer[0]); //Send complete UID (93 20) / SAK ( 93 70 + UID)
		}

		break;


	//
	// Receive RATS Command - See Section 5.1 Request for answer to select (ISO14443-4:2000)
	//
	case 0xE0:
		//
		// buffer[1] = Parameter byte - FSDI[7:4] CID[3:0]
		// buffer[2] = CRC1
		// buffer[3] = CRC2
		//
		//P2OUT |= BIT6 + BIT7;
		// Reset FIFO with an extra dummy clock
		buffer[0] = 0x00;
		RADIO_WRITERAW(RESET|BIT7, buffer, 1);

		buffer[0] = 0x06;	// Length of packet
		// ATS Packet
		buffer[1] = 6; // Length (Including itself, Not including CRC1 and CRC2)
		// TO (Format Byte):
		// BIT7 - Reserved for Future Use (RFU)
		// BIT6 - TA(1) Present
		// BIT6 - TB(1) Present
		// BIT4 - TC(1_ Present
		// BIT3:0 - FSCI = 0111, PICC capable of receiving 128 byte frames, from FSDI coding table
		buffer[2] = 0x77;
		// TA(1) Interface Byte Codes DS and DR ( Default value must be 0x00)
		// BIT7 - Unable to support different divisor for each direction (1= True, 0 = False)
		// BIT6 - DS8 Supported
		// BIT5 - DS4 Supported
		// BIT4 - DS2 Supported
		// BIT3 - Reserved for Future Use (RFU)
		// BIT2 - DR8 Supported
		// BIT1 - DR4 Supported
		// BIT0 - DR2 Supported
		buffer[3] = 0x00;
		//
		// TB(1) Interface Byte Codes FWI and SFGI
		//
		// BIT7:4 - FWI  => FWT  = (256 x 16 / fc) x 2 ^ FWI (See Section 7.2 ISO14443-4:2000)
		//        - FWI = 4 => FWT = 4.833 mS
		// BIT3:0 - SFGI => SFGT = (256 x 16 / fc) x 2 ^SFGI (See Section 5.2.5 ISO14443-4:2000)
		//        -SFGI = 2 => SFGT = 604 uS (See Section 6.1.4 ISO144443-3:2000)
		buffer[4] = 0x81;
		//
		// TC(1) Interface Byte Codes Protocol Options (See Section 5.2.6 ISO14443-4:2000)
		//
		// BIT7:2 - Shall be set to (000000)b, all other values are Reserved for Future Use (RFU)
		// BIT1   - CID Supported ( default to 1)
		// BIT0   - NAD Supported ( default to 0)
		buffer[5] = 0x02;

		//
		// Historical Data Byte - See 8ection 8.1.1. ISO7816:2000
		//
		buffer[6] = 0x80;		// Category Indicator Byte - A status indicator may be present in a COMPACT-TLV data object (one, two, or three bytes)

		NFCSend(&buffer[0]); //Send ATS Packet and wait for TX Complete

		break;

	//
	// Receive Protocol and Parameter Selection (PPS) Request - See Section 5.3 (ISO14443-4:2000)
	//
	case 0xD0:
		//
		// buffer[1] = Parameter 0 - Codes Presence of PPS1
		// buffer[2] = Parameter 1 - Codes DRI and DSI
		// buffer[3] = CRC1
		// buffer[4] = CRC2
		//

		// Reset FIFO with an extra dummy clock
		buffer[0] = 0x00;
		RADIO_WRITERAW(RESET|BIT7, buffer, 1);

		buffer[0] = 0x01;	// Length of packet
		// PPS Response Packet
		buffer[1] = 0xD0 | card_identifier;	// Based on the CID sent on the PPS Request

		NFCSend(&buffer[0]); //Send PPS Response Packet and wait for TX Complete

		break;
	// End of Type A Commands

	//
	// Type B Commands
	//

	//
	// Receive ATQB or WupB Req
	//
	case 0x05:
		// Reset FIFO with an extra dummy clock
		buffer[0] = 0x00;
		RADIO_WRITERAW(RESET|BIT7, buffer, 1);

		// ATQB Packet
		buffer[0] = 12; // Length
		buffer[1] = 0x50; // ATQB Start Byte
		// NFCID 3:0
		buffer[2] = TargetCID[0];
		buffer[3] = TargetCID[1];
		buffer[4] = TargetCID[2];
		buffer[5] = TargetCID[3];

		// Application Data Bytes
		buffer[6] = 0x40; // AFI - Public Telephony, GSM ... ( See ISO14443-3 Table 12 AFI Coding)
		buffer[7] = 0xE2; // CRC_B
		buffer[8] = 0xAF; // CRC_B
		buffer[9] = 0x11; // # of applications (1)

		// Protocol Info Bytes
		buffer[10] = 0x00; // Date Rate Capability ( Only Support 106 kbps)
		// Max Frame/Protocol type (128 bytes / PICC compliant to -4)
		buffer[11] = 0x71;
		// (FWI/ADC/FO) ( FWT = 77.3mSec, ADC = coded according to AFI, CID supported)
		buffer[12] = 0x85;

		NFCSend(&buffer[0]); //Send ATQB Packet and wait for TX Complete

		break;
	//
	// Receive ATTRIB Req
	//
	case 0x1D:
		if (buffer[1] == TargetCID[0] && buffer[2] == TargetCID[1]
				&& buffer[3] == TargetCID[2] && buffer[4] == TargetCID[3]) {
			// Reset FIFO with an extra dummy clock
			buffer[0] = 0x00;
			RADIO_WRITERAW(RESET|BIT7, buffer, 1);

			// ATTRIB Response
			buffer[0] = 0x01; // Length

			NFCSend(&buffer[0]); //Send ATTRIB Response and wait for TX Complete
		}

		break;
	//
	// Receive R(ACK) Packet - R-block containing a positive acknowledge (See ISO14443-4 Figure 15)
	//
	case 0xA2:
		// Reset FIFO with an extra dummy clock
		buffer[0] = 0x00;
		RADIO_WRITERAW(RESET|BIT7, buffer, 1);
		buffer[0] = 1;
		//
		// Response R(ACK) - See 7.5.4.3 ( PICC rules in ISO14443-4 Rule 12)
		//
		buffer[1] = 0xA2 | card_block_number;
		NFCSend(&buffer[0]);
		break;
	//
	// Receive R(NAK) Packet - R-block containing a positive acknowledge (See ISO14443-4 Figure 16)
	// Note: Reasons for R(NAK) - See PCD rules (7.5.4.2) ISO14443-4:2005
	//
	case 0xB2:
		// Reset FIFO with an extra dummy clock
		buffer[0] = 0x00;
		RADIO_WRITERAW(RESET|BIT7, buffer, 1);
		buffer[0] = 1;
		//
		// Response 0xA2
		//
		buffer[1] = 0xA2 | card_block_number;

		NFCSend(&buffer[0]);
		break;
	//
	// Receive a DESLECT request - respond 0x02 (See ISO14443-4 Table C1 - Block and frame coding)
	//
	case 0xC2: // deselect, section 8.8
		// Reset FIFO with an extra dummy clock
		buffer[0] = 0x00;
		RADIO_WRITERAW(RESET|BIT7, buffer, 1);
		buffer[0] = 1;		// Length
		buffer[1] = 0xC2;	// Error-free Deselect
		NFCSend(&buffer[0]);

//		P2OUT &= ~BIT5; //Turn off P2.5 as tag read is complete
		break;

	//
	// Protocol Control Byte = 0x02 | card_block_number (I-block PCD)
	//
	case 0x02:
		//
		// Received NDEF Tag Application Select Section 5.4.2, Type 4 Tag Operation Specification, Table 10, C-APDU).
		//
		if (buffer[2] == 0xA4) // container capability select command, section 8.2, figure 14 command#1
		{
			// Reset FIFO with an extra dummy clock
			buffer[0] = 0x00;
			RADIO_WRITERAW(RESET|BIT7, buffer, 1);
			//
			// Select NDEF Tag Application
			//
			if(buffer[3] == 0x04 && buffer[4] == 0x00 && buffer[5] == 0x07 && buffer[6] == 0xD2 && \
			   buffer[7] == 0x76 && buffer[10] == 0x85 && buffer[11] == 0x01)
			{
				SET_DEBUG_PIN_1;
				buffer[0] = 0x03; // Length
				buffer[1] = 0x02 | card_block_number;
				//
				// Command Completed - NDEF Tag Application Select  Detailed R-APDU Field Description (Table 15)
				//
				buffer[2] = 0x90; // SW1
				buffer[3] = 0x00; // SW2
			} else if(buffer[3] == 00 && buffer[4] == 0x0C && buffer[5] == 0x02 )
			{
				buffer[0] = 0x03; // Length
				buffer[1] = 0x02 | card_block_number;
				//
				// Command Completed - NDEF Tag Application Select  Detailed R-APDU Field Description (Table 15)
				//
				buffer[2] = 0x90; // SW1
				buffer[3] = 0x00; // SW2
			} else {
				buffer[0] = 0x03; // Length
				buffer[1] = 0x02 | card_block_number;

				//
				// Logical channel not supported (Table 6 ISO7816-4:2005))
				//
				buffer[2] = 0x6A; // SW1
				buffer[3] = 0x82; // SW2
			}

			NFCSend(&buffer[0]);


		}
		//
		// Received Capability Container Read Procedure ( Section 5.4.4 Table 16)
		//
		else if (buffer[2] == 0xB0 || buffer[2] == 0xB1) // read binary command, section 8.3, figure 16, command#3
		{
			// Reset FIFO with an extra dummy clock
			buffer[0] = 0x00;
			RADIO_WRITERAW(RESET|BIT7, buffer, 1);

			//
			// Request of the NDEF Message Length - (ReadBinary data from CC file See Section 5.4.4)
			//
			if( buffer[3] == 0x00 && buffer[4] == 0x00 && buffer[5] == 0x02)
			{
				buffer[0] = 5; // Length
				buffer[1] = 0x02 | card_block_number;
				//
				// Length of the NDEF Message :
				// (61 + g_device_name_length_reg) -  Simple Response
				// (75 + g_device_name_length_reg) - Static Handover (example as shown below)
				//
				if (TRANSMIT_SIMPLE_RESPONSE) {
					buffer[2] = ((58 + g_device_name_length_reg) & 0xFF00) >> 8; // NDEF File Length 15:8
					buffer[3] = (58 + g_device_name_length_reg) & 0xFF; // NDEF File Length 7:0
				} else if (TRANSMIT_STATIC_HANDOVER_RESPONSE) {
					buffer[2] = ((75 + g_device_name_length_reg) & 0xFF00) >> 8; // NDEF File Length 15:8
					buffer[3] = (75 + g_device_name_length_reg) & 0xFF; // NDEF File Length 7:0
				}
				//
				// Command Completed - NDEF Tag Application Select  Detailed R-APDU Field Description (Table 15)
				//
				buffer[4] = 0x90; // SW1
				buffer[5] = 0x00; // SW2
			}
			else if( buffer[5] == (58 + g_device_name_length_reg) )
			{
				// Ready to send NDEF Message
				buffer[0] = 61 + g_device_name_length_reg; // NDEF Message + 3 bytes
				buffer[1] = 0x02 | card_block_number;
				buffer[2] = ((56 + g_device_name_length_reg) & 0xFF00) >> 8; // NDEF Message Length - 2 bytes of length field [15:8]
				buffer[3] = (56 + g_device_name_length_reg) & 0xFF; // NDEF Message Length - 2 bytes of length field [7:0]
				buffer[4] = 0xD2; // NDEF Record Header: Mb=1b, ME=1b, CF=0b, SR=1b, IL=0b, TNF=001b

				buffer[5] = 0x20; // Record Type Length : 32 octets
				buffer[6] = 0x15 + g_device_name_length_reg; // OOB Data Length (See below buffer[39])
				//
				// Record Type Name : application/vdn.bluetooth.ep.oob
				//
				buffer[7] = 0x61; // 'a'
				buffer[8] = 0x70; // 'p'
				buffer[9] = 0x70; // 'p'
				buffer[10] = 0x6C; // 'l'
				buffer[11] = 0x69; // 'i'
				buffer[12] = 0x63; // 'c'
				buffer[13] = 0x61; // 'a'
				buffer[14] = 0x74; // 't'
				buffer[15] = 0x69; // 'i'
				buffer[16] = 0x6F; // 'o'
				buffer[17] = 0x6E; // 'n'
				buffer[18] = 0x2F; // '/'
				buffer[19] = 0x76; // 'v'
				buffer[20] = 0x6E; // 'd'
				buffer[21] = 0x64; // 'n'
				buffer[22] = 0x2E; // '.'
				buffer[23] = 0x62; // 'b'
				buffer[24] = 0x6C; // 'l'
				buffer[25] = 0x75; // 'u'
				buffer[26] = 0x65; // 'e'
				buffer[27] = 0x74; // 't'
				buffer[28] = 0x6F; // 'o'
				buffer[29] = 0x6F; // 'o'
				buffer[30] = 0x74; // 't'
				buffer[31] = 0x68; // 'h'
				buffer[32] = 0x2E; // '.'
				buffer[33] = 0x65; // 'e'
				buffer[34] = 0x70; // 'p'
				buffer[35] = 0x2E; // '.'
				buffer[36] = 0x6F; // 'o'
				buffer[37] = 0x6F; // 'o'
				buffer[38] = 0x62; // 'b'

				// Bluetooth OOB Data length Description:
				//	- BT ADDR (6 bytes)
				// 	- Class Device (5 bytes)
				// 	- Service Class UUID (6 bytes)
				// 	- Complete Local Name (g_device_name_length_reg + 2)
				// 	- Command Completed Response (2 bytes)
				buffer[39] = 0x15 + g_device_name_length_reg; // OOB Data Length
	//			buffer[39] = 0x08+temp_v; // OOB Data Length
				//
				// See Table 4 Binary Content of a Sample Bluetooth Handover Request in NFC Forum Bluetooth Simple Pairing Using NFC Document
				//
				buffer[40] = 0x00;
				buffer[41] = g_bt_address_reg[0]; // BT Address [7:0]
				buffer[42] = g_bt_address_reg[1]; // BT Address [15:8]
				buffer[43] = g_bt_address_reg[2]; // BT Address [23:16]
				buffer[44] = g_bt_address_reg[3]; // BT Address [31:24]
				buffer[45] = g_bt_address_reg[4]; // BT Address [39:32]
				buffer[46] = g_bt_address_reg[5]; // BT Address [47:40]

				buffer[47] = 0x04; // EIR Data Length
				buffer[48] = 0x0D; // EIR Data Type - Class of Device
				buffer[49] = 0x04; // Minor Device Class - Wearable Headset Device
				buffer[50] = 0x04; // Major Device class - Audio/Video
				buffer[51] = 0x20; // Service class - Audio

				buffer[52] = 0x05; // EIR Data Length
				buffer[53] = 0x03; // EIR Data Type - 16-bit Service Class UUID list (complete)
				buffer[54] = 0x0B; // A2DP-SNK [7:0]
				buffer[55] = 0x11; // A2DP-SNK [15:8]
				buffer[56] = 0x0F; // AVRCP-Controller [7:0]
				buffer[57] = 0x11; // AVRCP-Controller [15:8]

				buffer[58] = g_device_name_length_reg + 1; // EIR Data Length = Device Name Length + 1 EIR Data Type byte
				buffer[59] = 0x09; // EIR Data Type - Complete Local Name
				for (index_temp = 0; index_temp < g_device_name_length_reg;
						index_temp++) {
					//
					// Store the Bluetooth Local Name from g_device_name_reg
					//
					buffer[index_temp + 60] = g_device_name_reg[index_temp];
				}

				//
				// Command Completed - NDEF Tag Application Select  Detailed R-APDU Field Description (Table 15)
				//
				buffer[60 + g_device_name_length_reg] = 0x90; // SW1
				buffer[61 + g_device_name_length_reg] = 0x00; // SW2
			}
			//
			// If the length requested is 0x0F - Send the NDEF message length
			//
			else if( buffer[5] == 0x0F )
			{
				buffer[0] = 18; // Length
				buffer[1] = 0x02 | card_block_number; // Protocol Control Byte (PCB) (See ISO14443-4 Coding of I-block PCB)
				buffer[2] = 0x00; // CCLEN 15:8
				buffer[3] = 0x0F; // CCLEN 7:0
				buffer[4] = 0X20; // Mapping Version 2.0
				buffer[5] = 0X00; // Max R-ARPU size 15:8
				buffer[6] = 0x50+11; // Max R-ARPU size 7:0
				buffer[7] = 0X00; // Max C-APDU size 15:8
				buffer[8] = 0x50+11; // Max C-APDU size 7:0
				buffer[9] = 0X04; // T field of the NDEF File Control TLV
				buffer[10] = 0X06; // L field of the NDEF File Control TLV
				buffer[11] = 0XE1; // File identifier 15:8
				buffer[12] = 0x04; // File identifier 7:0
				buffer[13] = 0x08; // MAX NDEF file size 15:8
				buffer[14] = 0x00; // MAX NDEF file size 7:0
				buffer[15] = 0X00; // NDEF file read access condition; read access without any security
				buffer[16] = 0xff; // No write access granted at all (read-only) ( See section 5.1.2.1 NDEF File Control TLV)
				//
				// Command Completed - NDEF Tag Application Select  Detailed R-APDU Field Description (Table 15)
				//
				buffer[17] = 0X90; // SW1
				buffer[18] = 0X00; // SW2
			}
			//
			// If the length is 0x00 (the current EF requested)
			//
			else if( buffer[5] == 0x00 )
			{
				buffer[0] = 0x03; // Length
				buffer[1] = 0x02 | card_block_number;

				//
				// Logical channel not supported (Table 6 ISO7816-4:2005))
				//
				buffer[2] = 0x6A; // SW1
				buffer[3] = 0x82; // SW2
			}
			NFCSend(&buffer[0]); // sends read binary response, figure 17

		}
		//
		// Request of the NDEF Message Packet (Static Bluetooth Handover Command)- ReadBinary command ( See Section 5.4.6)
		//
		 else if (buffer[2] == 0xB0 && buffer[5] == (72 + g_device_name_length_reg))
		 {
			// Reset FIFO with an extra dummy clock
			buffer[0] = 0x00;
			RADIO_WRITERAW(RESET|BIT7, buffer, 1);
			// Ready to send NDEF Message
			buffer[0] = 75 + g_device_name_length_reg; // Total Length
			buffer[1] = 0x02 | card_block_number; //
			buffer[2] = ((70 + g_device_name_length_reg) & 0xFF00) >> 8; // NDEF Message Length [15:8]
			buffer[3] = (70 + g_device_name_length_reg) & 0xFF; // NDEF Message Length [7:0]
			buffer[4] = 0x91; // NDEF Record Header: Mb=1b, ME=0b, CF=0b, SR=1b, IL=0b, TNF=001b
			buffer[5] = 0x02; // Record Type Length : 2 octets
			buffer[6] = 0x0A; // Payload length : 10 octets
			//
			// Record Type: "Hs" - Handover Select Message
			//
			buffer[7] = 0x48; // 'H'
			buffer[8] = 0x73; // 'S'
			buffer[9] = 0x12; // Version Number : Major 1, Minor 2
			buffer[10] = 0xD1; // NDEF Record Header : Mb=1b, ME=1b, CF=0b, SR=1b, IL=0b, TNF=001b
			buffer[11] = 0x02; // Record Type Length : 2 octets
			buffer[12] = 0x04; // Payload Length: 4 octets
			//
			// Record Type: "ac" - Alternative Carrier
			//
			buffer[13] = 0x61; // 'a'
			buffer[14] = 0x63; // 'c'
			buffer[15] = 0x03; // Carrier Flags: CPS=?
			buffer[16] = 0x01; // Carrier Data Reference Length : 1 octet
			buffer[17] = 0x30; // Carrier Data Reference "0"
			buffer[18] = 0x00; // Auxiliary Data Reference Count : 0
			buffer[19] = 0x5A; // NDEF Record Header : Mb=0b, ME=1b, CF=0b, SR=1b, IL=1b, TNF=010b
			buffer[20] = 0x20; // Record Type Length : 32 octets
			buffer[21] = 0x15 + g_device_name_length_reg; // OOB Data Length (See below buffer[56])
			buffer[22] = 0x01; // Payload ID Length : 1 octet
			//
			// Record Type Name : application/vdn.bluetooth.ep.oob
			//
			buffer[23] = 0x61; // 'a'
			buffer[24] = 0x70; // 'p'
			buffer[25] = 0x70; // 'p'
			buffer[26] = 0x6C; // 'l'
			buffer[27] = 0x69; // 'i'
			buffer[28] = 0x63; // 'c'
			buffer[29] = 0x61; // 'a'
			buffer[30] = 0x74; // 't'
			buffer[31] = 0x69; // 'i'
			buffer[32] = 0x6F; // 'o'
			buffer[33] = 0x6E; // 'n'
			buffer[34] = 0x2F; // '/'
			buffer[35] = 0x76; // 'v'
			buffer[36] = 0x6E; // 'd'
			buffer[37] = 0x64; // 'n'
			buffer[38] = 0x2E; // '.'
			buffer[39] = 0x62; // 'b'
			buffer[40] = 0x6C; // 'l'
			buffer[41] = 0x75; // 'u'
			buffer[42] = 0x65; // 'e'
			buffer[43] = 0x74; // 't'
			buffer[44] = 0x6F; // 'o'
			buffer[45] = 0x6F; // 'o'
			buffer[46] = 0x74; // 't'
			buffer[47] = 0x68; // 'h'
			buffer[48] = 0x2E; // '.'
			buffer[49] = 0x65; // 'e'
			buffer[50] = 0x70; // 'p'
			buffer[51] = 0x2E; // '.'
			buffer[52] = 0x6F; // 'o'
			buffer[53] = 0x6F; // 'o'
			buffer[54] = 0x62; // 'b'
			buffer[55] = 0x30; // Payload ID: "0"
			// Bluetooth OOB Data length Description:
			//	- BT ADDR (6 bytes)
			// 	- Class Device (5 bytes)
			// 	- Service Class UUID (6 bytes)
			// 	- Complete Local Name (g_device_name_length_reg + 2)
			// 	- Command Completed Response (2 bytes)
			buffer[56] = 0x15 + g_device_name_length_reg; // OOB Data Length
			//
			// See Table 4 Binary Content of a Sample Bluetooth Handover Request in NFC Forum Bluetooth Simple Pairing Using NFC Document
			//
			buffer[57] = 0x00;
			buffer[58] = g_bt_address_reg[0]; // BT Address [7:0]
			buffer[59] = g_bt_address_reg[1]; // BT Address [15:8]
			buffer[60] = g_bt_address_reg[2]; // BT Address [23:16]
			buffer[61] = g_bt_address_reg[3]; // BT Address [31:24]
			buffer[62] = g_bt_address_reg[4]; // BT Address [39:32]
			buffer[63] = g_bt_address_reg[5]; // BT Address [47:40]

			buffer[64] = 0x04; // EIR Data Length
			buffer[65] = 0x0D; // EIR Data Type - Class of Device
			buffer[66] = 0x04; // Minor Device Class - Wearable Headset Device
			buffer[67] = 0x04; // Major Device class - Audio/Video
			buffer[68] = 0x20; // Service class - Audio

			buffer[69] = 0x05; // EIR Data Length
			buffer[70] = 0x03; // EIR Data Type - 16-bit Service Class UUID list (complete)
			buffer[71] = 0x0B; // A2DP-SNK [7:0]
			buffer[72] = 0x11; // A2DP-SNK [15:8]
			buffer[73] = 0x0F; // AVRCP-Controller [7:0]
			buffer[74] = 0x11; // AVRCP-Controller [15:8]

			buffer[75] = g_device_name_length_reg + 1; // EIR Data Length = Device Name Length + 1 EIR Data Type byte
			buffer[76] = 0x09; // EIR Data Type - Complete Local Name
			for (index_temp = 0; index_temp < g_device_name_length_reg; index_temp++) {
				//
				// Store the Bluetooth Local Name from g_device_name_reg
				//
				buffer[index_temp + 77] = g_device_name_reg[index_temp];
			}

			//
			// Command Completed - NDEF Tag Application Select  Detailed R-APDU Field Description (Table 15)
			//
			buffer[77 + g_device_name_length_reg] = 0x90; // SW1
			buffer[78 + g_device_name_length_reg] = 0x00; // SW2

			NFCSend(&buffer[0]); // sends read binary response, figure 27
		}
		//
		// Proximity Coupling Device (PCD) sending a proprietary card select
		//
		else if (buffer[2] == 0x5A || buffer[1] == 0xB8) {
			// Reset FIFO with an extra dummy clock
			buffer[0] = 0x00;
			RADIO_WRITERAW(RESET|BIT7, buffer, 1);
			buffer[0] = 0x03; // Length
			buffer[1] = 0x02 | card_block_number;
			//
			// NDEF Tag Application not found; no data is returned (Table 12 NFC Forum Type 4 Operation Spec)
			//
//			buffer[2] = 0x91;
//			buffer[3] = 0x00;
			buffer[2] = 0x69;
			buffer[3] = 0x86;
			NFCSend(&buffer[0]);
		}
		else if(buffer[1] == 0x70)
		{
			// Reset FIFO with an extra dummy clock
			buffer[0] = 0x00;
			RADIO_WRITERAW(RESET|BIT7, buffer, 1);
			buffer[0] = 0x03; // Length
			buffer[1] = 0x02 | card_block_number;
			//
			// Chainning not supported
			//
			buffer[2] = 0x68;
			buffer[3] = 0x83;
			NFCSend(&buffer[0]);
		}
		else
		{
			//
			// The PCD is attempting to connect to another application
			//

			// Reset FIFO with an extra dummy clock
			buffer[0] = 0x00;
			RADIO_WRITERAW(RESET|BIT7, buffer, 1);
//			buffer[0] = 0x03; // Length
//			buffer[1] = 0x02 | card_block_number;
//			//
//			// NDEF Tag Application not found; no data is returned (Table 12 NFC Forum Type 4 Operation Spec)
//			//
//			buffer[2] = 0x62;
//			buffer[3] = 0x83;
//			NFCSend(&buffer[0]);
		}

		break;
	default :
		// Reset FIFO with an extra dummy clock
		buffer[0] = 0x00;
		RADIO_WRITERAW(RESET|BIT7, buffer, 1);
		break;
	}

	return ret;
}

NFC_state_t NFCTarget_statemachine(void)
{
	switch (NFC_State) {
	case POWER_OFF_STATE:
		Trf797xInitialSettings();
		NFC_State = SENSE_STATE;

		break;
	case SENSE_STATE:
//		CLEAR_DEBUG_PIN_1;
//		CLEAR_DEBUG_PIN_2;
		NFC_State = SENSE_STATE;

		u8InterruptFlags = INT_BUSY;
		do {
			Radio_Wait_IRQ();
		} while (u8InterruptFlags != INT_RX_COMPLETE && NFC_State == SENSE_STATE);

		if (u8InterruptFlags == INT_RX_COMPLETE) {
			NFC_State = EMULATION_STATE;
		}

		else if (NFC_State != EMULATION_STATE)
		{
			break;
		}
	case EMULATION_STATE:
//		SET_DEBUG_PIN_1;
		// Now wait until the part is selected
		// restart the pointer and wait for a message
		do {
			if ((u8InterruptFlags
					& (INT_RX_COMPLETE | INT_CRITICAL_ERROR | INT_COMM_ERROR))
					== INT_RX_COMPLETE)
				NFC_ProcessReceivedData(&buf[0]); // Check if the message was a ATR_REQ
			else if ((u8InterruptFlags & INT_CRITICAL_ERROR))
				NFC_State = POWER_OFF_STATE; // Timeout Error

			do {
				// Wait for RX Complete interrupt
				Radio_Wait_IRQ();
			} while (u8InterruptFlags != INT_RX_COMPLETE && (NFC_State == EMULATION_STATE ));
		} while (NFC_State == EMULATION_STATE);
		__no_operation();
		break;
	}
	return NFC_State;
}

//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
void main(void) {
	unsigned volatile counter = 0;

	//
	// Initialize Version Number in INFO Block at address 0x01040
	//
	WDTCTL = WDTPW + WDTHOLD; // Stop WDT

	TRF_DISABLE;
	// Set EN to output direction
	ENABLE_SET;
	// wait until system clock started
	McuDelayMillisecond(10);
	// Set EN = 1 on the TRF 79xx
	TRF_ENABLE;
	//BT_WAKE signal config and initialization
//	ENABLE_DEBUG_PIN_1;
	//
	// Clear the debug pins
	//
//	CLEAR_DEBUG_PIN_1;

	// wait until system clock started
	McuDelayMillisecond(1);


	//
	// Enable an interrupt Low to High transition for the IRQ pin on the TRF7970A
	//
	IRQ_PIN_SET;
	P2IE |= 0x02;                             // P2.1 interrupt enabled
	P2IES &= ~0x02;                           // P2.1 Low to High edge
	P2IFG &= ~0x02;                           // P2.1 IFG cleared


//	//********************************************************************
//	//********************************************************************
//	//DLP - set external antenna
//	P1DIR |= BIT1;
//	//P1OUT |= BIT1; //internal antenna
//	P1OUT &= ~BIT1; //external antenna
//	//********************************************************************
//	//********************************************************************


	// set the oscillator
	McuOscSel(1);

	//
	// Initializing communication module with TRF7970 ( Parallel or SPI)
	//
	Trf797xCommunicationSetup();//forced to parallel

	McuDelayMillisecond(10);

	//
	// Power GPIO - P1.7 Turn On
	//
	P1DIR |= BIT7;
	P1OUT |= BIT7;

	//
	// Enable and set corresponding LED to ISO14443 TYPE A / B
	//
#if CARD_EMULATION_TYPE_A
	P1DIR |= BIT6;
	P1OUT |= BIT6;
#endif

#if CARD_EMULATION_TYPE_B
	P1DIR |= BIT5;
	P1OUT |= BIT5;
#endif

	// General enable interrupts
	__bis_SR_register(GIE);

	// Set up an interrupt for high to low transition on the SDA line of I2C

	for (;;) {
		// Initialize TRF7970 in Card Emulation
		NFCTarget_statemachine();
	}
}


#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR(void)                            // interrupt handler
{
	//
	// Clear the interrupt flag
	//
	P2IFG &= ~0x02;
//	TRF_ENABLE;
	__bic_SR_register_on_exit(LPM3_bits);
}


