/****************************************************************
* FILENAME: trf796x.c
*
* BRIEF: Contains functions to initialize and execute
* communication the TRF796x via the selected interface.
*
* Copyright (C) 2010 Texas Instruments, Inc.
*
* AUTHOR(S): Reiser Peter        DATE: 02 DEC 2010
*
* EDITED BY:
* *
*
****************************************************************/

#include "trf796x.h"
#include "parallel.h"

//===============================================================

u08_t    command[2];
u08_t    direct_mode = 0;


extern u08_t    i_reg;

extern u08_t    irq_flag;
extern u08_t    rx_error_flag;
extern s08_t    rxtx_state;
extern u08_t    stand_alone_flag;

u08_t TargetCID[7] = TARGET_CID;

//===============================================================

void Trf797xISR(u08_t *irq_status);

//===============================================================
//                                                              ;
//===============================================================

void
Trf797xCommunicationSetup(void)
{
	if(SPIMODE)                                // SPI mode given by jumper setting
	{
		SpiSetup();
	}
	else									// parallel mode
	{
		ParallelSetup();
	}
}

//===============================================================
// NAME: void Trf796xDirectCommand (u08_t *pbuf)
//
// BRIEF: Is used to transmit a Direct Command to TRF796x.
//
// INPUTS:
//    Parameters:
//        u08_t        *pbuf        Direct Command
//
// OUTPUTS:
//
// PROCESS:    [1] transmit Direct Command
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf797xDirectCommand(u08_t *pbuf)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiDirectCommand(pbuf);
  }
  else									// parallel mode
  	{
  		ParallelDirectCommand(pbuf);
  	}
}

//===============================================================
// NAME: void Trf796xDirectMode (void)
//
// BRIEF: Is used to start Direct Mode.
//
// INPUTS:
//
// OUTPUTS:
//
// PROCESS:    [1] start Direct Mode
//
// NOTE: No stop condition
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf797xDirectMode(void)
{
  direct_mode = 1;
  
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiDirectMode();
  }
  else									// parallel mode
  	{
  		ParallelDirectMode();
  	}
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xDisableSlotCounter(void)
{
  buf[40] = IRQ_MASK;
  buf[41] = IRQ_MASK;                // next slot counter
  Trf797xReadSingle(&buf[41], 1);
  buf[41] &= 0xFE;                // clear BIT0 in register 0x01
  Trf797xWriteSingle(&buf[40], 2);
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xEnableSlotCounter(void)
{
  buf[40] = IRQ_MASK;
  buf[41] = IRQ_MASK;                // next slot counter
  Trf797xReadSingle (&buf[41], 1);
  buf[41] |= BIT0;                // set BIT0 in register 0x01
  Trf797xWriteSingle(&buf[40], 2);
}




//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================


void Radio_Transfer(u08_t mode_addr, u08_t *pbuf, u08_t length)
{
    u16_t i;
    u08_t *txbuf, *rxbuf;
        
    for (i=length; i > 0; i--)
    {
        if (!(mode_addr & BIT6))    // If writing, readjust buff
           pbuf[i] = pbuf[i-1];
        else if (i != 0)          // When reading, clear buffer
          pbuf[i] = 0x00;
    }
    pbuf[0] = mode_addr;

    rxbuf = txbuf = pbuf; // Initialize pointer

    SLAVE_SELECT_LOW; //Start SPI Mode
    
     // USCI_A0 TX buffer ready?
    while (!(IFG2 & TRF7970A_TX_IFG_FLAG));
    
    // Previous data to TX, RX
    TRF7970A_SPI_TX_BUF = *txbuf++;
    
    while(TRF7970A_SPI_STAT & UCBUSY);    // Ensure USCI is not busy

    while(length > 0)
    {
    	TRF7970A_SPI_TX_BUF = *txbuf++;
    

        while(UCB0STAT & UCBUSY);    // Ensure USCI is not busy
        *rxbuf++ = TRF7970A_SPI_RX_BUF;
        length--;
    }
    
    while(TRF7970A_SPI_STAT & UCBUSY);    // Ensure USCI is not busy

    SLAVE_SELECT_HIGH; //Stop SPI Mode
}

void Radio_InitialSettings(void)
{
//   RADIO_DIRECTCOMMAND(SOFT_INIT, buf);
//	buf[0] = MODULATOR_CONTROL;
//	buf[1] = 0x21;
//	ParallelWriteSingle(buf,1);
}

void
Trf797xReaderSettings(void)
{
  command[0] = SOFT_INIT;
  Trf797xDirectCommand(command);
  
  command[0] = IDLE;
  Trf797xDirectCommand(command);
}

void
Trf797xInitialSettings(void)
{
	unsigned int i;

//	SET_DEBUG_PIN_2;

#if CARD_EMULATION_TYPE_A
	//
	// Type A Initialization
	//

	// Software Init Command
	RADIO_DIRECTCOMMAND(SOFT_INIT, buf);

	// Idle Command
	RADIO_DIRECTCOMMAND(IDLE, buf);

	McuDelayMillisecond(3);

	// Register 00h. Chip Status Control Register
	buf[0] = 0x21;
	//
	// Transmitter on - 3 V Operation
	//
	RADIO_WRITESINGLE(CHIP_STATE_CONTROL, buf);

	// Register 01h. ISO Control Register
	buf[0] = 0x24; // Type A
	//
	// 106 kbps Tag Emulator 14443A mode
	//
	RADIO_WRITESINGLE(ISO_CONTROL, buf);

	// Reset FIFO with an extra dummy clock
	buf[0] = 0x00;
	RADIO_WRITERAW(RESET|BIT7, buf,1);

	// Register 02h. ISO14443B TX Options Register
	buf[0] = 0x01;
#if 1
	//
	// AutoSDD disable
	//
	buf[0] = 0x00;
#endif
	//
	// Auto SDD configuration (ISO14443A Layer 4 compliant)
	//
	RADIO_WRITESINGLE(ISO_14443B_OPTIONS, buf);

	// Register 09h. Modulator and SYS_CLK Control Register
	// Option 1:
	// BIT6 - OOK Modulation Enabled
	// Option 2:
	// BIT0 - OOK(100 )
	buf[0] = 0x01;
	//
	// No SYS_CLK  Output, Modulation OOK(100%)
	//
	RADIO_WRITESINGLE(MODULATOR_CONTROL, buf);

	// Register 0Ah. RX Special Settings Register
	buf[0] = 0x30;
	//
	// BIT 5 - Bandpass 450 kHz to 1.5 MHz - Appropiate for Manchester-coded 848 kHz subcarrier used in ISO14443A/B
	// BIT 4 - Bandpass 100 kHz to 1.5 MHz - Appropiate for highest bit rate (848 kbps) used in high-bit-rate-ISO14443
	//
	RADIO_WRITESINGLE(RX_SPECIAL_SETTINGS, buf);

	// Register 0Bh. RX Special Setting Register
	buf[0] = 0x87;
	//
	// Automatic Regulator Setting
	//
	RADIO_WRITESINGLE(REGULATOR_CONTROL,buf);

	// Register 14h. FIFO IRQ level Register
	buf[0] = 0x0F;
	//
	// Set FIFO IRQ Level for RX High = 96 bytes and TX low 32 bytes
	//
	RADIO_WRITESINGLE(FIFO_IRQ_LEVEL, buf);

	// Register 16h. NFC Low Field Detection Level Register
	buf[0] = 0x83; // Type A Card
	//
	// BIT 7 - Disable clock extractor - NFC passive 106 kbps and ISO14443A card emulation
	// BIT 2:0 = RF field level for RF collision avoidance ( displayed in B6 of register 0x19)
	//
	RADIO_WRITESINGLE(NFC_LOW_DETECTION, buf);

	// Register 17h. NFCID1 Number Register
	for ( i=0; i < 4; i++)
		buf[i] = TargetCID[i];
	//
	// NFCID Number (4, 7, or 10 bytes)
	//
	RADIO_WRITECONT(NFCID, &buf[0], 4);

	// Register 18h. NFC Target Detection level Register
	buf[0] = 0x27;     // Type A
#if 1
	//
	// AutoSDD disable
	//
	buf[0] = 0x07;
#endif
	//
	// BIT 7:6 - NFC Target Detection Level (NFCID size = 7)
	// BIT 5 - Automatic SDD using internal state machine
	// BIT 2:0 = RF field level required for system wakeup - displayed in B7 of register 0x19)
	//
	RADIO_WRITESINGLE(NFC_TARGET_LEVEL, buf);

//    // Register 1Ah. Test Register (for Test or Direct Use)
	buf[0] = 0x40;
	//
	// Set first RX gain stage
	//
	RADIO_WRITESINGLE(NFC_TEST_SETTING1_REG, buf);

	// Reset FIFO with an extra dummy clock
	buf[0] = 0x00;
	RADIO_WRITERAW(RESET|BIT7, buf,1);

	// Block Receiver, then Enable Receiver
	   //--Resetting Decoders--//
	RADIO_DIRECTCOMMAND(STOP_DECODERS, buf);
	RADIO_DIRECTCOMMAND(RUN_DECODERS, buf);

	//
	// Read all the registers - Ensure Registers were initialized properly
	//
//    RADIO_READCONT(CHIP_STATE_CONTROL,buf,0x17);
//    RADIO_READCONT(NFC_TARGET_LEVEL,buf,0x4);
    #endif

#if CARD_EMULATION_TYPE_B
    //
    // Type B Initialization
    //

    // Software Init Command
//	RADIO_DIRECTCOMMAND(SOFT_INIT, buf);
	buf[0] = SOFT_INIT;
	ParallelDirectCommand(buf);

	// Idle Command
//	RADIO_DIRECTCOMMAND(IDLE, buf);
	buf[0] = IDLE;
	ParallelDirectCommand(buf);
//
	McuDelayMillisecond(3);



	// Register 00h. Chip Status Control Register
//	buf[0] = 0x20;
//
//	buf[0] = MODULATOR_CONTROL;
//	ParallelReadSingle(buf,0x01);
//
//	for(i=0;i < 8; i++)
//	{
//		buf[0] = NFC_LOW_DETECTION;
//		buf[1] = i;
//		ParallelWriteSingle(buf,1);
//
//		buf[0] = NFC_LOW_DETECTION;
//		ParallelReadSingle(buf,0x01);
//	}

	//
	// Transmitter on - 3 V Operation
	//
//	RADIO_WRITESINGLE(CHIP_STATE_CONTROL, buf);
	buf[0] = CHIP_STATE_CONTROL;
	buf[1] = 0x21;
	ParallelWriteSingle(buf,1);

	// Register 01h. ISO Control Register
//	buf[0] = 0x25; // Type B
	//
	// 106 kbps Tag Emulator 14443A mode
	//
//	RADIO_WRITESINGLE(ISO_CONTROL, buf);
	buf[0] = ISO_CONTROL;
	buf[1] = 0x25;
	ParallelWriteSingle(buf,1);

	// Reset FIFO with an extra dummy clock
//	buf[0] = 0x00;
//	RADIO_WRITERAW(RESET|BIT7, buf,1);
	buf[0] = RESET|BIT7;
	buf[1] = 0x00;
	ParallelRawWrite(buf,2);

	// Register 02h. ISO14443B TX Options Register
//	buf[0] = 0x00;
	//
	// Auto SDD configuration Disabled
	//
//	RADIO_WRITESINGLE(ISO_14443B_OPTIONS, buf);
	buf[0] = ISO_14443B_OPTIONS;
	buf[1] = 0x00;
	ParallelWriteSingle(buf,1);

	// Register 09h. Modulator and SYS_CLK Control Register
	// Option 1:
	// BIT6 - OOK Modulation Enabled
	// Option 2:
	// BIT0 - OOK(100 )
	// 0x00 = ASK 10 %
//	buf[0] = 0x00;
	//
	// No SYS_CLK  Output, Modulation OOK(100%)
	//
//	RADIO_WRITESINGLE(MODULATOR_CONTROL, buf);
	buf[0] = MODULATOR_CONTROL;
	buf[1] = 0x00;
	ParallelWriteSingle(buf,1);

	// Register 0Ah. RX Special Settings Register
//	buf[0] = 0x30;
	//
	// BIT 5 - Bandpass 450 kHz to 1.5 MHz - Appropiate for Manchester-coded 848 kHz subcarrier used in ISO14443A/B
	// BIT 4 - Bandpass 100 kHz to 1.5 MHz - Appropiate for highest bit rate (848 kbps) used in high-bit-rate-ISO14443
	//
//	RADIO_WRITESINGLE(RX_SPECIAL_SETTINGS, buf);
	buf[0] = RX_SPECIAL_SETTINGS;
	buf[1] = 0x30;
	ParallelWriteSingle(buf,1);

	// Register 0Bh. RX Special Setting Register
//	buf[0] = 0x87;
	//
	// Automatic Regulator Setting
	//
//	RADIO_WRITESINGLE(REGULATOR_CONTROL,buf);
	buf[0] = REGULATOR_CONTROL;
	buf[1] = 0x87;
	ParallelWriteSingle(buf,1);

	// Register 14h. FIFO IRQ level Register
//	buf[0] = 0x0F;
	//
	// Set FIFO IRQ Level for RX High = 96 bytes and TX low 32 bytes
	//
//	RADIO_WRITESINGLE(FIFO_IRQ_LEVEL, buf);
	buf[0] = FIFO_IRQ_LEVEL;
	buf[1] = 0x0F;
	ParallelWriteSingle(buf,1);

	// Register 16h. NFC Low Field Detection Level Register
//	buf[0] = 0x03; // Type A Card
	//
	// BIT 2:0 = RF field level for RF collision avoidance ( displayed in B6 of register 0x19)
	//
//	RADIO_WRITESINGLE(NFC_LOW_DETECTION, buf);
	buf[0] = NFC_LOW_DETECTION;
	buf[1] = 0x04;
	ParallelWriteSingle(buf,1);

	buf[0] = NFCID;
	// Register 17h. NFCID1 Number Register
	for ( i=1; i < 5; i++)
		buf[i] = TargetCID[i-1];
	//
	// NFCID Number (4, 7, or 10 bytes)
	//
//	RADIO_WRITECONT(NFCID, &buf[0], 4);
	ParallelWriteCont(buf,5);

	// Register 18h. NFC Target Detection level Register
//	buf[0] = 0x07;     // Type B
	//
	// BIT 7:6 - 00b NFC Target Detection Level (NFCID size = 4)
	// BIT 2:0 = RF field level required for system wakeup - displayed in B7 of register 0x19)

	RADIO_WRITESINGLE(NFC_TARGET_LEVEL, buf);
	buf[0] = NFC_TARGET_LEVEL;
	buf[1] = 0x07;
	ParallelWriteSingle(buf,1);

//    // Register 1Ah. Test Register (for Test or Direct Use)
//	buf[0] = 0x40;
	//
	// Set first RX gain stage
	//
//	RADIO_WRITESINGLE(NFC_TEST_SETTING1_REG, buf);
	buf[0] = NFC_TEST_SETTING1_REG;
	buf[1] = 0x40;
	ParallelWriteSingle(buf,1);

	// Reset FIFO with an extra dummy clock
//	buf[0] = 0x00;
//	RADIO_WRITERAW(RESET|BIT7, buf,1);
	buf[0] = RESET|BIT7;
	buf[1] = 0x00;
	ParallelRawWrite(buf,2);


	// Block Receiver, then Enable Receiver
	   //--Resetting Decoders--//
//	RADIO_DIRECTCOMMAND(STOP_DECODERS, buf);
//	RADIO_DIRECTCOMMAND(RUN_DECODERS, buf);

	buf[0] = STOP_DECODERS;
	ParallelDirectCommand(buf);
	buf[0] = RUN_DECODERS;
	ParallelDirectCommand(buf);

	buf[0] = CHIP_STATE_CONTROL;
	ParallelReadCont(buf,0x17);
//	buf[0] = NFC_TARGET_LEVEL;
//	ParallelReadCont(buf,0x04);


	//
	// Read all the registers - Ensure Registers were initialized properly
	//
//    RADIO_READCONT(CHIP_STATE_CONTROL,buf,0x17);
//    RADIO_READCONT(NFC_TARGET_LEVEL,buf,0x4);
//}

#endif
}

//===============================================================
// NAME: void Trf796xRawWrite (u08_t *pbuf, u08_t length)
//
// BRIEF: Is used to write direct to the TRF796x.
//
// INPUTS:
//    Parameters:
//        u08_t        *pbuf        raw data
//        u08_t        length        number of data bytes
//
// OUTPUTS:
//
// PROCESS:    [1] send raw data to TRF796x
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf797xRawWrite(u08_t *pbuf, u08_t length)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiRawWrite(pbuf, length);
  }
  else									// parallel mode
  	{
  		ParallelRawWrite(pbuf, length);
  	}
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xReConfig(void)
{
  if(SPIMODE)
  {
    SpiUsciExtClkSet();                   // Set the USART
  }
}

//===============================================================
// NAME: void Trf796xReadCont (u08_t *pbuf, u08_t length)
//
// BRIEF: Is used to read a specified number of TRF796x registers
// from a specified address upwards.
//
// INPUTS:
//    Parameters:
//        u08_t        *pbuf        address of first register
//        u08_t        length        number of registers
//
// OUTPUTS:
//
// PROCESS:    [1] read registers
//            [2] write contents to *pbuf
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf797xReadCont(u08_t *pbuf, u08_t length)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiReadCont(pbuf, length);
  }
  else									// parallel mode
  	{
  		ParallelReadCont(pbuf, length);
  	}
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xReadIrqStatus(u08_t *pbuf)
{
  *pbuf = IRQ_STATUS;
  *(pbuf + 1) = IRQ_MASK;
  //Trf796xReadCont(pbuf, 2);           // read second reg. as dummy read
  Trf797xReadSingle(pbuf, 1); //parallel
}

//===============================================================
// NAME: void Trf796xReadSingle (u08_t *pbuf, u08_t number)
//
// BRIEF: Is used to read specified TRF796x registers.
//
// INPUTS:
//    Parameters:
//        u08_t        *pbuf        addresses of the registers
//        u08_t        number        number of the registers
//
// OUTPUTS:
//
// PROCESS:    [1] read registers
//            [2] write contents to *pbuf
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf797xReadSingle(u08_t *pbuf, u08_t number)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiReadSingle(pbuf, number);
  }
  else									// parallel mode
  	{
  		ParallelReadSingle(pbuf, number);
  	}
}

//===============================================================
// resets FIFO                                                  ;
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xReset(void)

{
  command[0] = RESET;
  Trf797xDirectCommand(command);
  
}

//===============================================================
// resets IRQ Status                                            ;
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xResetIrqStatus(void)
{
  u08_t irq_status[4];
  irq_status[0] = IRQ_STATUS;
  irq_status[1] = IRQ_MASK;
  
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    Trf797xReadCont(irq_status, 2);        // read second reg. as dummy read
  }
  else									// parallel mode
  	{
  		Trf797xReadSingle(irq_status, 1);
  	}
}

//===============================================================
//                                                              ;
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xRunDecoders(void)
{
  command[0] = RUN_DECODERS;
  Trf797xDirectCommand(command);
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xStopDecoders(void)
{
  command[0] = STOP_DECODERS;
  Trf797xDirectCommand(command);
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xTransmitNextSlot(void)
{
  command[0] = TRANSMIT_NEXT_SLOT;
  Trf797xDirectCommand(command);
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xTurnRfOff(void)
{
  command[0] = CHIP_STATE_CONTROL;
  command[1] = CHIP_STATE_CONTROL;
  Trf797xReadSingle(&command[1], 1);
  command[1] &= 0x1F;
  Trf797xWriteSingle(command, 2);
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xTurnRfOn(void)
{
  u08_t write[4];
  write[0] = CHIP_STATE_CONTROL;
  write[1] = 0x20;                    // 3.3VDC, Full power out
  Trf797xWriteSingle(write, 2);
  
}

//===============================================================
// NAME: void SpiWriteCont (u08_t *pbuf, u08_t length)
//
// BRIEF: Is used to write to a specific number of reader chip
// registers from a specific address upwards.
//
// INPUTS:
//    u08_t    *pbuf    address of first register followed by the
//                    contents to write
//    u08_t    length    number of registers + 1
//
// OUTPUTS:
//
// PROCESS:    [1] write to the registers
//
// CHANGE:
// DATE          WHO    DETAIL
//===============================================================

void
Trf797xWriteCont(u08_t *pbuf, u08_t length)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiWriteCont(pbuf, length);
  }
  else									// parallel mode
  	{
  		ParallelWriteCont(pbuf, length);
  	}
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf797xWriteIsoControl(u08_t iso_control)
{
  u08_t write[16];
  
  write[0] = ISO_CONTROL;
  write[1] = iso_control;
  write[1] &= ~BIT5;
  Trf797xWriteSingle(write, 2);
  
  iso_control &= 0x1F;
  write[0] = IRQ_MASK;
  write[1] = 0x3F;
  Trf797xWriteSingle(write, 2);
}

//===============================================================
// NAME: void Trf796xWriteSingle (u08_t *pbuf, u08_t length)
//
// BRIEF: Is used to write to specified reader chip registers.
//
// INPUTS:
//    u08_t    *pbuf    addresses of the registers followed by the
//                    contents to write
//    u08_t    length    number of registers * 2
//
// OUTPUTS:
//
// PROCESS:    [1] write to the registers
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf797xWriteSingle(u08_t *pbuf, u08_t length)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiWriteSingle(pbuf, length);
  }
  else									// parallel mode
  	{
  		ParallelWriteSingle(pbuf, length);
  	}
}
